#include <stdlib.h>
#include <stdio.h>
#include "geometry.h"

Point2 *
makePoint2 (double x, double y)
{
  Point2 *p = (Point2 *) malloc (sizeof (Point2));
  #ifdef DEBUG 
  	printf("allocatign point %x\n", p);
  #endif
  p->x = x;
  p->y = y;
  return p;
}

double
det3x3 (Point3 a, Point3 b, Point3 c)
{
  // Calcula o determinante 3x3
  return a.x * (b.y * c.z - c.y * b.z) - a.y * (b.x * c.z - c.x * b.z) +
    a.z * (b.x * c.y - c.x * b.y);

}

double
det2x2 (Point2 a, Point2 b)
{
	return a.x * b.y - b.x * a.y;
}

double 
delta (Point2 a, Point2 b, Point2 c)
{
	return det2x2(b,c) - det2x2(a,c) + det2x2(a,b);
}

int leftTurn(Point2 a, Point2 b, Point2 c)
{
	double det;
	det = delta(a,b,c);
	if (det != 0.) return det > 0.;
	det = c.y - b.y;
	if (det != 0.) return det < 0.;
	det = c.x - b.x;
	if (det != 0.) return det < 0.;
	det = c.y - a.y;
	if (det != 0.) return det > 0.;
	return 0;
}

double
tao (Point3 a, Point3 b, Point3 c, Point3 d)
{
  // Calcula o deterinante 4x4
  return det3x3 (b, c, d) - det3x3 (a, c, d) + det3x3 (a, b, d) - det3x3 (a,
									  b,
									  c);
}

// Verifica se o ponto d está no triângulo definido por abc
int
incircle (Point2 a, Point2 b, Point2 c, Point2 d)
{
  // Calcula determinante delta
  double delta =
    (b.x * c.y - b.y * c.x) - (a.x * c.y - a.y * c.x) + (a.x * b.y -
							 a.y * b.x);
  // Calcula as coordenadas adicionais (pontos 3)
  Point3 a3, b3, c3, d3;
  a3.x = a.x;
  a3.y = a.y;
  a3.z = a.x * a.x + a.y * a.y;
  b3.x = b.x;
  b3.y = b.y;
  b3.z = b.x * b.x + b.y * b.y;
  c3.x = c.x;
  c3.y = c.y;
  c3.z = c.x * c.x + c.y * c.y;
  d3.x = d.x;
  d3.y = d.y;
  d3.z = d.x * d.x + d.y * d.y;
  // Calcula determinante tao
  double t = tao (a3, b3, c3, d3);
  // Devolve a multiplicação delta * tao
  return (t * delta) < 0;
}

int
inTriangle (Point2 * a, Point2 * b, Point2 * c, Point2 * d)
{
	int val = leftTurn(*a,*b,*d) + leftTurn(*b,*c,*d) + leftTurn(*c,*a,*d);
	return val == 0 || val == 3;
}

Point2 *
barycenter (Point2 * a, Point2 * b, Point2 * c)
{
  return makePoint2 (1. / 3. * a->x + 1. / 3. * b->x + 1. / 3. * c->x,
		     1. / 3. * a->y + 1. / 3. * b->y + 1. / 3. * c->y);
}

void freePoint2(Point2 *p)
{
	if (p == NULL) return;
	#ifdef DEBUG 
		printf("freeing point %x\n", p); 
	#endif
	free(p);
	p = NULL;
}
