#include <stdio.h>
#include "svg.h"

#define K 50
#define S 1 

void printHeader() {
	printf ("<?xml version=\"1.0\" standalone=\"no\"?>\n");
  printf ("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\n");
  printf ("\"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n");
  printf ("<svg id=\"root\" width=\"100%%\" height=\"100%%\" version=\"1.1\"\n");
  printf ("xmlns=\"http://www.w3.org/2000/svg\" onload=\"draw();\">\n");
  printf ("<script type=\"text/ecmascript\">\n");
  printf ("<![CDATA[\n");
printf("function draw() {\n");
printf("var svgNS = \"http://www.w3.org/2000/svg\";\n");
printf("var list = new Array();\n");
printf("var svg = document.getElementById(\"root\");\n");

}

void printArtificialTriangle(Triangle *t) {
	printGroupBegin("artificialTriangle");
	drawEdge(t->e1, "green");
	drawEdge(t->e2, "green");
	drawEdge(t->e3, "green");
	printGroupEnd();
	
}

void printSplit(Edge *e1, Edge *e2, Edge *e3) {
	static int split = 0;
	char buffer[50000];
	sprintf(buffer, "split%d", split++);
	printGroupBegin(buffer);
	drawEdge(e1, "green");
	drawEdge(e2, "green");
	drawEdge(e3, "green");
	printGroupEnd();
	
}

void printFlip(Edge *e1, Edge *e2) {
	static int flip = 0;
	char buffer[50000];
	sprintf(buffer, "flip%d", flip++);
	// Apaga a primeira aresta e coloca a segunda
	printGroupBegin(buffer);
	eraseEdge(e1);
	drawEdge(e2, "green");
	printGroupEnd();
	
}

void printIllegal(Edge *e) {
	static int illegal = 0;
	char buffer[50000];
	sprintf(buffer, "illegal%d", illegal++);
	// Apaga a primeira aresta e coloca a segunda
	printGroupBegin(buffer);
	changeEdgeColor(e, "red");
	printGroupEnd();
	
}

void printTriangle(Triangle *t, char* color)
{
	static int nt = 0;
	  printf ("<polygon name=\"triangle%d\" points=\"", nt++);
      printf ("%.0f,%.0f ", S * (t->v1->point->x + K), S * (t->v1->point->y + K));
      printf ("%.0f,%.0f ", S * (t->v2->point->x + K), S * (t->v2->point->y + K));
      printf ("%.0f,%.0f ", S * (t->v3->point->x + K), S * (t->v3->point->y + K));
      printf ("\" style=\"fill:white;stroke:%s;\"/>\n", color);
}
void drawEdge(Edge *e, char* color)
{
	double x1 = S * (e->v1->point->x + K);
	double y1 = S * (e->v1->point->y + K);
	double x2 = S * (e->v2->point->x + K);
	double y2 = S * (e->v2->point->y + K);
	printf("e = document.createElementNS(svgNS, \"line\");\n");
	printf("e.setAttributeNS(null, \"id\", \"line_%.0f_%.0f_%.0f_%.0f_\");\n", x1, y1, x2, y2);
	printf("e.setAttributeNS(null, \"x1\", %.0f);\n", x1);
	printf("e.setAttributeNS(null, \"y1\", %.0f);\n", y1);
	printf("e.setAttributeNS(null, \"x2\", %.0f);\n", x2);
	printf("e.setAttributeNS(null, \"y2\", %.0f);\n", y2);
	printf("e.setAttributeNS(null, \"style\", \"stroke:%s\");\n", color);
	printf("g.appendChild(e);\n");
	
}
void eraseEdge(Edge *e)
{
	double x1 = S * (e->v1->point->x + K);
	double y1 = S * (e->v1->point->y + K);
	double x2 = S * (e->v2->point->x + K);
	double y2 = S * (e->v2->point->y + K);
	printf("e = document.getElementById(\"line_%.0f_%.0f_%.0f_%.0f_\");\n", x1, y1, x2, y2);
	printf("e.setAttributeNS(null, \"style\", \"display:none;\");\n");
	
}

void changeEdgeColor(Edge *e, char *color)
{
	double x1 = S * (e->v1->point->x + K);
	double y1 = S * (e->v1->point->y + K);
	double x2 = S * (e->v2->point->x + K);
	double y2 = S * (e->v2->point->y + K);
	printf("e = document.getElementById(\"line_%.0f_%.0f_%.0f_%.0f_\");\n", x1, y1, x2, y2);
	printf("e.setAttributeNS(null, \"style\", \"stroke:%s;\");\n", color);
	
}

void printVertex(Vertex *v, char* color)
{
	static int nv = 0;
	printf("<circle id=\"vertex%d\" cx='%.0f' cy='%.0f' r='1' stroke='black' fill='red' />",nv++,
		S * (v->point->x + K), S * (v->point->y + K));
}

void printGroupBegin(char *str) {
	printf("list.push(function %s() {\n", str);
	printf("g = document.createElementNS(svgNS, \"g\");\n");
	printf("g.setAttributeNS(null, \"id\", \"%s\");\n", str);
}

void printGroupEnd() {
	printf("svg.appendChild(g);\n});\n\n");
}

void printFooter() {
	int time = 100;
	  printf ("for (i = 0; i < list.length; i++) {\n");
	  printf ("\twindow.setTimeout(list[i], i * %d);\n", time);
	  printf ("}\n");
	  printf ("}\n//]]>\n");
	  printf ("</script>\n");
	  printf ("<g id=\"root\" />\n");
	  printf ("</svg>\n");
}
