#include <stdlib.h>
#include <stdio.h>
#include "topology.h"

Vertex *
getOpposedVertex (Edge * e, Triangle * t)
{
  // Recuperar o triângulo oposto o ao triângulo t na aresta e
  Triangle *opposed = getOpposedTriangle (e, t);

  if (opposed == NULL)
	return NULL;

  // Recuperar o vértice do triângulo o que não está em e
  if (opposed->v1 != e->v1 && opposed->v1 != e->v2)
    return opposed->v1;
  else if (opposed->v2 != e->v1 && opposed->v2 != e->v2)
    return opposed->v2;
  else if (opposed->v3 != e->v1 && opposed->v3 != e->v2)
    return opposed->v3;
  else
    return NULL;
}

Edge *
getTriangleEdge (Vertex * v1, Vertex * v2, Triangle * t)
{
  if ((t->e1->v1 == v1 || t->e1->v2 == v1)
      && (t->e1->v1 == v2 || t->e1->v2 == v2))
    return t->e1;
  if ((t->e2->v1 == v1 || t->e2->v2 == v1)
      && (t->e2->v1 == v2 || t->e2->v2 == v2))
    return t->e2;
  if ((t->e3->v1 == v1 || t->e3->v2 == v1)
      && (t->e3->v1 == v2 || t->e3->v2 == v2))
    return t->e3;

  return 0;
}

Triangle *
getTriangle (Edge * e, Vertex * v)
{
  Vertex *ov = getOpposedVertex (e, e->t1);
  if (ov == NULL || v != ov)
  {
    return e->t1;
  }
  else
  {
    return e->t2;
  }
}

Triangle *
getOpposedTriangle (Edge * e, Triangle * t)
{
  // Recuperar o triângulo oposto o ao triângulo t na aresta e
  Triangle *opposed;
  if (t == e->t1)
    opposed = e->t2;
  else
    opposed = e->t1;

  return opposed;
}

Vertex *
makeVertex (Point2* point)
{
  Vertex *v = (Vertex *)malloc (sizeof (Vertex));
  #ifdef DEBUG
  	printf("allocating vertex %x\n", v);
  #endif
  v->point = point;
  v->artificial = 0;
  return v;
}


Edge *
makeEdge (Vertex * v1, Vertex * v2)
{
  Edge *e = (Edge *) malloc (sizeof (Edge));
  #ifdef DEBUG
  	printf("allocating edge %x\n", e);
  #endif
  e->v1 = v1;
  e->v2 = v2;
  e->t1 = NULL;
  e->t2 = NULL;
  
  return e;
}


Triangle *
makeTriangle (Vertex * v1, Vertex * v2, Vertex * v3)
{
  Triangle *t = (Triangle *)malloc (sizeof (Triangle));
  #ifdef DEBUG
  	printf("allocating triangle %x\n", t);
  #endif
  t->v1 = v1;
  t->v2 = v2;
  t->v3 = v3;
  t->e1 = NULL;
  t->e2 = NULL;
  t->e3 = NULL;
  
  return t;
}

void freeTriangle(Triangle *t)
{
	if (t->e1 == NULL && t->e2 == NULL && t->e3 == NULL) return;
	if (t->v1 == NULL && t->v2 == NULL && t->v3 == NULL) return;
	if (t == NULL) return;
	#ifdef DEBUG
		printf("freeing triangle %x\n", t);
	#endif
	freeEdge(t->e1);
	freeEdge(t->e2);
	freeEdge(t->e3);
	t->e1 = t->e2 = t->e3 = NULL;
	freeVertex(t->v1);
	freeVertex(t->v2);
	freeVertex(t->v3);
	t->v1 = t->v2 = t->v3 = NULL;
	free(t);
	t = NULL;
}

void freeEdge(Edge *e)
{
	if (e->v1 == NULL && e->v2 == NULL) return;
	if (e == NULL) return;
	#ifdef DEBUG
		printf("freeing edge %x\n", e);
	#endif
	free(e);
	e->v1 = e->v2 = NULL;
	e = NULL;
}

void freeVertex(Vertex *v)
{
	if (v->point == NULL) return;
	#ifdef DEBUG
		printf("freeing vertex %x\n", v);
	#endif
	freePoint2(v->point);
	v->point = NULL;
	v = NULL;
}
